#!/usr/local/bin/perl

######################################################
# requires route2country_20001015.txt for ASFinder   #
######################################################

use strict;

use vars qw ($asDataFile $netgeo $asFinder);
use Socket;
use CAIDA::ASFinder;

if($#ARGV != 1)
{
    print "Usage: buildDataFile_part1.pl <inputFile> <outputFile>\n";
    die("Invalid Argument count");
}

$asDataFile = 'route2country_20001015.txt';
$asFinder = new CAIDA::ASFinder;
print "Loading AS data\n";
$asFinder->load_file_text($asDataFile);


my $inputFile = $ARGV[0];
my $outputFile = $ARGV[1];

print "Reading from file \"$inputFile\"\n";
my $hostHash = readInputFile($inputFile);
my $key;

foreach $key (keys %$hostHash)
{
    $$hostHash{$key} = getAS($key);

    #print "-"x30 . "\n";
    #print "Hostname : $key\n";
    #print "AS       : " . $$hostHash{$key} . "\n";
}
#print "-"x30 . "\n";

writeOutput($outputFile, $hostHash);



sub getAS 
{
    my ($ip) = @_;
    if(!($ip =~ /\d{1,3}\.\d{1,3}\.\d{1,3}\,\d{1,3}/))
    {
	$ip = getIP($ip);
    }
    #print "IP      : $ip\n";

    my $as_num = $asFinder->get_as($ip);
    if(!defined($as_num)) 
    {
	$as_num = "not_found";
    } 
    elsif ($as_num =~ /MultipleOrigins/) 
    {
	$as_num = "not_found";
    }
    return $as_num;
}

sub getIP 
{
    my $hostname = $_[0];
    my $iaddr = gethostbyname($hostname);
    my $ip;
    if(defined $iaddr)
    {
	$ip = inet_ntoa($iaddr);
    }
    else
    {
	$ip = "not_found";
    }
    return $ip;
}

#
# Stuff specific to our input file format
#
sub readInputFile
{
    my ($file_name) = @_;
    my (%hostHash, $readingHostnames);
    $readingHostnames = 0;

    open (INPUT_F, "<$file_name");

    while(<INPUT_F>)
    {

	if(/^\s*Total/ || /^\s*Path/)
	{
	    $readingHostnames = 0;
	}

	if($readingHostnames == 1)
	{
	    s/^\s*(\S(.*\S)?)\s*$/$1/;
	    $hostHash{$_} = 1;
	    #print "Adding hostname " . $_ . "\n";
	}

	if(/^\s*Path/)
	{
	    $readingHostnames = 1;
	}
    }

    close(INPUT_F);

    return \%hostHash;
}

sub writeOutput
{
    my ($file_name, $hostHash) = @_;
    open(OUTPUT_F, ">$file_name");
    my $key;
    foreach $key (sort keys %$hostHash)
    {
	print OUTPUT_F "$key\t$$hostHash{$key}\n";
    }
    close(OUTPUT_F);
}
