//==========================================================================
//
// Copyright 2002 The Regents of the University of California
// All Rights Reserved
//
// Permission to use, copy, modify and distribute any part of this Makefile
// for educational, research and non-profit purposes, without fee, and
// without a written agreement is hereby granted, provided that the above 
// copyright notice, this paragraph and the following paragraphs appear 
// in all copies.
//
// Those desiring to incorporate this into commercial products or use for
// commercial purposes should contact the
//
//      Technology Transfer Office, University of California, San Diego,
//      9500 Gilman Drive, La Jolla, CA 92093-0910
//      Ph: (619) 534-5815, FAX: (619) 534-7345.
//
// IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS PLOTPATH, EVEN 
// IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF 
// SUCH DAMAGE.
//
// THE PLOTPATH, PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
// UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
// SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY OF 
// CALIFORNIA PLOTPATH NO REPRESENTATIONS AND EXTENDS NO WARRANTIES OF 
// ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED TO, 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
// PURPOSE, OR THAT THE USE OF THE SKITTERTRACES WILL NOT INFRINGE ANY 
// PATENT, TRADEMARK OR OTHER RIGHTS.
//
// PlotPaths is developed by John Gallagher Bradley L. Huffaker at the
// University of California, San Diego under the Cooperative Association
// for Internet Data Analysis (CAIDA) Program. Support for this effort is
// provided by NSF "Internet Atlas" grant ANI-I-9996248, NSF "CAIDA Core"
// NCR-9711092, and by CAIDA memebers.
//==========================================================================
package org.caida.otter;

/**
 * A class representing a bounding box with explicit coordinates.
 */
public class BoundingBox {

    /**
     * Create a bounding box with these boundaries
     *
     * @param x1 The first x val generally describing the left of the box.
     * @param y1 The first y val generally describing the bottom of the box.
     * @param x2 The second x val generally describing the right of the box.
     * @param y2 The secind y val generally describing the top of the box.
     */
    public BoundingBox(int x1, int y1, int x2, int y2) {
	leftBound = x1;
	upperBound = y1;
	rightBound = x2;
	lowerBound = y2;
    }

    /**
     * Test the provided line segment for intersection with this bounding box.
     *   The parameters list two points describing a line segement such that
     *   a given line segment goes from point (x1,y1) to (x2,y2).
     *
     * @return true if the line intersects this bounding box, false otherwise.
     */
    public boolean doesLineCrossBoundingBox(int x1, int y1, int x2, int y2) {

	boolean result_value = false;

	LineSegment query = new LineSegment(x1, y1, x2, y2);

	LineSegment bbox1 = new LineSegment(leftBound, 
					    lowerBound, 
					    leftBound, 
					    upperBound);

	LineSegment bbox2 = new LineSegment(leftBound, 
					    upperBound, 
					    rightBound, 
					    upperBound); 

	LineSegment bbox3 = new LineSegment(rightBound, 
					    upperBound, 
					    rightBound, 
					    lowerBound);

	LineSegment bbox4 = new LineSegment(rightBound, 
					    lowerBound, 
					    leftBound, 
					    lowerBound);

	if(query.doesLineIntersect(bbox1)) {
	    result_value = true;

	} else if (query.doesLineIntersect(bbox2)) {
	    result_value = true;

	} else if (query.doesLineIntersect(bbox3)) {
	    result_value = true;

	} else if (query.doesLineIntersect(bbox4)) {
	    result_value = true;
	}

	return result_value;
    } 

    /**
     * Get the top boundary
     *
     * @return The top of the box (y2)
     */
    public int getTop() {
	return upperBound;
    }

    /**
     * Get the left boundary
     *
     * @return The left of the box (x1)
     */
    public int getLeft() {
	return leftBound;
    }

    /**
     * Get the bottom boundary
     *
     * @return The bottom of the box (y1)
     */
    public int getBottom() {
	return lowerBound;
    }

    /**
     * Get the right boundary
     *
     * @return The right of the box (x2)
     */
    public int getRight() {
	return rightBound;
    }

    public void debug() {
	System.out.println("============ begin BoundingBox =========");
	System.out.println("[Debug] Bounding Box");
	System.out.println("upperBound: "+upperBound);
	System.out.println("lowerBound: "+lowerBound);
	System.out.println("leftBound: "+leftBound);
	System.out.println("rightBound: "+rightBound);
	System.out.println("============ end BoundingBox ===========");
    }
    private int upperBound;
    private int lowerBound;
    private int leftBound;
    private int rightBound;
}
