//==========================================================================
//
// Copyright 2002 The Regents of the University of California
// All Rights Reserved
//
// Permission to use, copy, modify and distribute any part of this Makefile
// for educational, research and non-profit purposes, without fee, and
// without a written agreement is hereby granted, provided that the above 
// copyright notice, this paragraph and the following paragraphs appear 
// in all copies.
//
// Those desiring to incorporate this into commercial products or use for
// commercial purposes should contact the
//
//      Technology Transfer Office, University of California, San Diego,
//      9500 Gilman Drive, La Jolla, CA 92093-0910
//      Ph: (619) 534-5815, FAX: (619) 534-7345.
//
// IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS PLOTPATH, EVEN 
// IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF 
// SUCH DAMAGE.
//
// THE PLOTPATH, PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
// UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
// SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY OF 
// CALIFORNIA PLOTPATH NO REPRESENTATIONS AND EXTENDS NO WARRANTIES OF 
// ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED TO, 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
// PURPOSE, OR THAT THE USE OF THE SKITTERTRACES WILL NOT INFRINGE ANY 
// PATENT, TRADEMARK OR OTHER RIGHTS.
//
// PlotPaths is developed by John Gallagher Bradley L. Huffaker at the
// University of California, San Diego under the Cooperative Association
// for Internet Data Analysis (CAIDA) Program. Support for this effort is
// provided by NSF "Internet Atlas" grant ANI-I-9996248, NSF "CAIDA Core"
// NCR-9711092, and by CAIDA memebers.
//==========================================================================
package org.caida.otter;

/**
 * Base class for a lot of display type thingys
 */
public class Coordinate {

    public String name = "Unnamed";

    public boolean isSetXY = false;
    public int x = -1;
    public int y = -1;

    public boolean isSetColRow = false;
    public int colNum = -1;
    public int rowNum = -1;
    public String colName = null;

    public int index;
    //Modified by John Tusson Tue Aug 14 16:20:16 PDT 2001
    public int nodeNumber;

    /**
     * Create a new coordinate
     */
    public Coordinate(int index) {
	this.index 	= index;
    }

    /**
     * Create a new coordinate
     */
    public Coordinate(int index, int col, int row) {
	this.index 	= index;
	this.colNum 	= col;
	this.rowNum	= row;
    }

    /**
     * Create a new coordinate
     */
    public Coordinate(int index, int col, int row, String name) {
	this.index	= index;
	this.colNum 	= col;
	this.rowNum 	= row;
	this.name 	= name;
    }

    /**
     * Create a new coordinate
     */
    public Coordinate(int index, int col, int row, String name, String colName) {
	this.index	= index;
	this.colNum 	= col;
	this.rowNum 	= row;
	this.name 	= name;
	this.colName	= colName;
    }

    /**
     *  Keep track of the orignal node number from
     *  the LibSea Graph file.  John Tusson Tue Aug 14 16:23:29 PDT 2001
     */
    public Coordinate(int index, int col, int row, String name, String colName, int nodeNumber)
    {
	this.index	= index;
	this.colNum 	= col;
	this.rowNum 	= row;
	this.name 	= name;
	this.colName	= colName;
	this.nodeNumber = nodeNumber;
    }

    /**
     * set column name
     */
    public void setColumnName(String colName) {
	this.colName = colName;
    }

    /**
     * get column name
     */
    public String getColumnName() {
	return colName;
    }

    /**
     * set column number
     */
    public void setColumnNum(int colNum) {
	this.colNum 	= colNum;
    }

    /**
     * set the row number
     */
    public void setRowNum(int rowNum) {
	this.rowNum 	= rowNum;
    }

    /**
     * Get the column number
     */
    public int getColNum() {
	return colNum;
    }

    /**
     * Get the row number
     */
    public int getRowNum() {
	return rowNum;
    }

    /**
     * Get the coordinate name
     */
    public String getName() {
	return name;
    }

    /**
     * Set the node name
     */
    public void setName(String name) {
	this.name = name;
    }

    /**
     * override default equals to do name equivalance if both args are
     *   Coordinates.
     */
    public boolean equals(Object otherObj) {
	if(otherObj instanceof Coordinate) {
	    if(this.name.equals(((Coordinate)otherObj).name)) {
		return true;
	    } else {
		return false;
	    }
	} else {
	    return otherObj.equals(this);
	}
    }

    public void debug() {
	System.out.println("============ begin Coordinate ============");
	System.out.println("[Debug] Coordinate");
	System.out.println("index: "+index);
	System.out.println("x: "+x);
	System.out.println("y: "+y);
	System.out.println("isSetXY: "+isSetXY);
	System.out.println("colName: "+colName);
	System.out.println("colNum: "+colNum);
	System.out.println("rowNum: "+rowNum);
	System.out.println("isSetColRow: "+isSetColRow);
	System.out.println("============ end Coordinate ==============");
    }
}
