//==========================================================================
//
// Copyright 2002 The Regents of the University of California
// All Rights Reserved
//
// Permission to use, copy, modify and distribute any part of this Makefile
// for educational, research and non-profit purposes, without fee, and
// without a written agreement is hereby granted, provided that the above 
// copyright notice, this paragraph and the following paragraphs appear 
// in all copies.
//
// Those desiring to incorporate this into commercial products or use for
// commercial purposes should contact the
//
//      Technology Transfer Office, University of California, San Diego,
//      9500 Gilman Drive, La Jolla, CA 92093-0910
//      Ph: (619) 534-5815, FAX: (619) 534-7345.
//
// IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS PLOTPATH, EVEN 
// IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF 
// SUCH DAMAGE.
//
// THE PLOTPATH, PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
// UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
// SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY OF 
// CALIFORNIA PLOTPATH NO REPRESENTATIONS AND EXTENDS NO WARRANTIES OF 
// ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED TO, 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
// PURPOSE, OR THAT THE USE OF THE SKITTERTRACES WILL NOT INFRINGE ANY 
// PATENT, TRADEMARK OR OTHER RIGHTS.
//
// PlotPaths is developed by John Gallagher Bradley L. Huffaker at the
// University of California, San Diego under the Cooperative Association
// for Internet Data Analysis (CAIDA) Program. Support for this effort is
// provided by NSF "Internet Atlas" grant ANI-I-9996248, NSF "CAIDA Core"
// NCR-9711092, and by CAIDA memebers.
//==========================================================================
package org.caida.otter;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Comparator;

/**
 * A container for nodes, labels, and links.  This class should encapsulate
 *   all the necessary information to render a graph to the screen.
 */
public class DisplayObjects {

    /**
     * Create an empty display object
     */
    public DisplayObjects() {
	lines	= new Hashtable();
	nodes	= new Hashtable();
	labels 	= new Hashtable();
    }

    public void addColumnNameToNodeName() {
	Node tmpNode;
	String colName;

	Enumeration nodeEnum = getNodeEnumeration();
	while(nodeEnum.hasMoreElements()) {
	    tmpNode = (Node)nodeEnum.nextElement();
	    colName = tmpNode.getColumnName();
	    if(colName.equals("")) {
		colName = "No Data";
		tmpNode.setColumnName(colName);
	    }
	    tmpNode.setName(tmpNode.getName()+" ("+colName+")");
	}
    }

    /**
     * check the data structure for self-consistency
     */
    public boolean checkSelfConsistencyForNodesAndLines() {

	// make sure all neighbors of nodes exist

	// make sure the endpoints of all lines exist
	return true;
    }
    
    /**
     * Add a node to our DisplayObject
     *
     * @param index The index value to assign the new node.
     * @param column The column number to put the new node in.
     * @param row The row number to put the new node in.
     * @param name The name to assign the new node.
     * @param colName The column name to assign the new node.
     * @return The newly created Node object.
     */
    public Node addNode(int index, int column, int row, String name, 
				String colName) 
    {
	Node newNode = new Node(index, column, row, name, colName);
	Object tmp = nodes.put(name, newNode);
	if(tmp != null)
	{
	    System.out.println("Overwriting node: \"" + name + "\"");
	}

	return newNode;
    }

    /**
     * Add a node to the DisplayObject
     * John Tusson Tue Aug 14 16:28:12 PDT 2001
     * @param index The index value to assign the new node.
     * @param column The column number to put the new node in.
     * @param row The row number to put the new node in.
     * @param name The name to assign the new node.
     * @param colName The column name to assign the new node.
     * @param nodeNumber The orignal node number from graph file
     * @return The newly created Node object.
     */
    public Node addNode(int index, int column, int row, String name, 
				String colName, int nodeNumber) 
    {

	Node newNode = new Node(index, column, row, name, colName, nodeNumber);
	Object tmp = nodes.put(name, newNode);
	if(tmp != null)
	{
	    System.out.println("Overwriting node: \"" + name + "\"");
	}
	return newNode;
    }
    
    public Node addNode(Node newNode) {
	Object tmp = nodes.put(newNode.getName(), newNode);
	if(tmp != null)
	{
	    System.out.println("Overwriting node: \"" + newNode.getName() + 
		"\"");
	}
	return newNode;
    }

    public Node getNode(String name) {
	return (Node)nodes.get(name);
    }

    public Enumeration getNodeEnumeration() {
	return nodes.elements();
    }

    public boolean containsNode(Node node) {
	if(nodes.containsValue(node)) {
	    return true;
	} else {
	    return false;
	}
    }
    
    public boolean containsNode(String nodeName) {
	if(nodes.containsKey(nodeName)) {
	    return true;
	} else {
	    return false;
	}
    }

    public int getNodeCount() {
	return nodes.size();
    }

    public int getLabelCount() {
	return labels.size();
    }

    public int getLineCount() {
	return lines.size();
    }


    /**
     * Add a label to our DisplayObject
     * 
     * @param index The index to assign the new Label
     * @param column The column number to assign the new label.
     * @param row The row number to assign the new label.
     * @param name The name to assign the new label.
     * @param colName The column name to assign to the new label.
     * @return The newly created Label object.
     */
    public Label addLabel(int index, int column, int row, String name, 
				String colName) {
	Label newLabel = new Label(index, column, row, name, colName);
	Object tmp = labels.put(name, newLabel);
	if(tmp != null)
	{
	    System.out.println("Overwriting label: \"" + name + "\"");
	}
	return newLabel;
    }

    public Label addLabel(Label newLabel) {
	Object tmp = labels.put(newLabel.getName(), newLabel);
	if(tmp != null)
	{
	    System.out.println("Overwriting label: \"" + newLabel.getName() + 
		"\"");
	}
	return newLabel;
    }
 
    public Label getLabel(String name) {
	return (Label)labels.get(name);
    }

    public Enumeration getLabelEnumeration() {
	return labels.elements();
    }

    public boolean containsLabel(Label label) {
	if(labels.containsValue(label)) {
	    return true;
	} else {
	    return false;
	}
    }

    public boolean containsLabel(String labelName) {
	if(labels.containsKey(labelName)) {
	    return true;
	} else {
	    return false;
	}
    }

    /**
     * Add a line to our DisplayObject
     *
     * @param idx1 the index of the node the line starts from.
     * @param idx2 the index of the node the line ends at.
     * @param name A name to assign to the new line.
     * @return The newly created Line object.
     */
    public Line addLine(Coordinate coord1, Coordinate coord2, String name) {
	Line newLine = new Line(coord1, coord2, name);
	Object tmp = lines.put(name, newLine);
	if(tmp != null)
	{
	    System.out.println("Overwriting line: \"" + newLine.getName() + 
		"\"");
	}

	return newLine;
    }

    public Line addLine(Line newLine) {
	Object tmp = lines.put(newLine.getName(), newLine);
	if(tmp != null) {
	    System.out.println("Overwriting line: \"" + newLine.getName() + 
		"\"");
	}

	return newLine;
    }

    public Line getLine(String name) {
	return (Line)lines.get(name);
    }

    public Enumeration getLineEnumeration() {
	return lines.elements();
    }

    public boolean containsLine(Line line) {
	if(lines.containsValue(line)) {
	    return true;
	} else {
	    return false;
	}
    }

    public boolean containsKey(String lineName) {
	if(lines.containsKey(lineName)) {
	    return true;
	} else {
	    return false;
	}
    }



    /**
     * Invert our display object along either axis.
     *
     * @param invertX true if you want the X axis inverted, false otherwise.
     * @param invertY true if you want the Y axis inverted, false otherwise.
     */
    public void invertImage(boolean invertX, boolean invertY) {
	
	//////////////////////////////
	// find our max dimensions  //
	//////////////////////////////

	if(!invertX && !invertY) {
	    return;
	}

	int max_y;
	int max_x;
	int min_y;
	int min_x;

	Node tmpNode;
	Label tmpLabel;

	Enumeration nodeEnum = nodes.elements();
	Enumeration labelEnum = labels.elements();

	if(nodeEnum.hasMoreElements()) {
	    tmpNode = (Node)nodeEnum.nextElement();
	    max_x = tmpNode.x;
	    max_y = tmpNode.y;
	    min_x = tmpNode.x;
	    min_y = tmpNode.y;
	} else if(labelEnum.hasMoreElements()) {
	    tmpLabel = (Label)labelEnum.nextElement();
	    max_x = tmpLabel.x;
	    max_y = tmpLabel.y;
	    min_x = tmpLabel.x;
	    min_y = tmpLabel.y;
	} else {
	    return;
	}


	// find the min and max coordinates
	while(nodeEnum.hasMoreElements()) {
	    tmpNode = (Node)nodeEnum.nextElement();
	    if(invertX) {
		if(tmpNode.x < min_x) {
		    min_x = tmpNode.x;
		}
		if(tmpNode.x > max_x) {
		    max_x = tmpNode.x;
		}
	    }
	    if(invertY) {
		if(tmpNode.y < min_y) {
		    min_y = tmpNode.y;
		}
		if(tmpNode.y > max_y) {
		    max_y = tmpNode.y;
		}
	    }
	}
	while(labelEnum.hasMoreElements()) {
	    tmpLabel = (Label)labelEnum.nextElement();
	    if(invertX) {
		if(tmpLabel.x < min_x) {
		    min_x = tmpLabel.x;
		}
		if(tmpLabel.x > max_x) {
		    max_x = tmpLabel.x;
		}
	    }
	    if(invertY) {
		if(tmpLabel.y < min_y) {
		    min_y = tmpLabel.y;
		}
		if(tmpLabel.y > max_y) {
		    max_y = tmpLabel.y;
		}
	    }
	}

	// get new enumerations to visit all the elements again
	nodeEnum = nodes.elements();
	labelEnum = labels.elements();

	// apply the changes after finding min and max coordinates
	while(nodeEnum.hasMoreElements()) {
	    tmpNode = (Node)nodeEnum.nextElement();
	    if(invertX) {
		tmpNode.x = max_x - (tmpNode.x - min_x);
	    }
	    if(invertY) {
		tmpNode.y = max_y - (tmpNode.y - min_y);
	    }
	}
	while(labelEnum.hasMoreElements()) {
	    tmpLabel = (Label)labelEnum.nextElement();
	    if(invertX) {
		tmpLabel.x = max_x - (tmpLabel.x - min_x);
	    }
	    if(invertY) {
		tmpLabel.y = max_y - (tmpLabel.y - min_y);
	    }
	}
    }

    /**
     * Remove a label from the DisplayObject
     *
     * @param delLabel The label object to remove.
     */
    public void removeLabel(Label delLabel) {

	Label tmpLabel;
	if(labels.containsValue(delLabel)) {
	    labels.remove(delLabel.getName());
	}
    }

    /**
     * Normalize the size of the final image.  An argument of -1 for 
     *   either width or height will leave that axis unchanged.
     *
     * @param screenX The new width in pixels.
     * @param screenY the new height in pixels.
     */
    public void normalizeSize(int screenX, int screenY) {

	if(screenX == -1 && screenY == -1) {
	    // they don't want the picture normalized
	    return;
	}

	// find our maximum bounds
	int minX;
	int maxX;

	int minY;
	int maxY;

	Node tmpNode;
	Label tmpLabel;

	Enumeration nodeEnum = nodes.elements();
	Enumeration labelEnum = labels.elements();

	if(nodeEnum.hasMoreElements()) {
	    tmpNode = (Node)nodeEnum.nextElement();
	    minX = tmpNode.x;
	    maxX = tmpNode.x;
	    minY = tmpNode.y;
	    maxY = tmpNode.y;
	} else if (labelEnum.hasMoreElements()) {
	    tmpLabel = (Label)labelEnum.nextElement();
	    minX = tmpLabel.x;
	    maxX = tmpLabel.x;
	    minY = tmpLabel.y;
	    maxY = tmpLabel.y;
	} else {
	    // (nothing to sort)
	    return;
	}

	while(nodeEnum.hasMoreElements()) {
	    tmpNode = (Node)nodeEnum.nextElement();
	    if(minX > tmpNode.x) {
		minX = tmpNode.x;
	    }
	    if(maxX < tmpNode.x) {
		maxX = tmpNode.x;
	    }
	    if(minY > tmpNode.y) {
		minY = tmpNode.y;
	    }
	    if(maxY < tmpNode.y) {
		maxY = tmpNode.y;
	    }
	}
	while(labelEnum.hasMoreElements()) {
	    tmpLabel = (Label)labelEnum.nextElement();
	    if(minX > tmpLabel.x) {
		minX = tmpLabel.x;
	    }
	    if(maxX < tmpLabel.x) {
		maxX = tmpLabel.x;
	    }
	    if(minY > tmpLabel.y) {
		minY = tmpLabel.y;
	    }
	    if(maxY < tmpLabel.y) {
		maxY = tmpLabel.y;
	    }
	}

	System.out.println("minX: "+minX+" maxX: "+maxX);
	System.out.println("minY: "+minY+" maxY: "+maxY);

	// calculate conversion factors
	int offsetX = 0;
	int offsetY = 0;
	double conversionX = 1;
	double conversionY = 1;

	if(screenX != -1) {
	    offsetX = minX * -1;
	    if((maxX + offsetX) == 0) {
		conversionX = 1;
	    } else {
		conversionX = (double)screenX / (double)(maxX + offsetX);
	    }
	}

	if(screenY != -1) {
	    offsetY = minY * -1;
	    if((maxY + offsetY) == 0) {
		conversionY = 1;
	    } else {
		conversionY = (double)screenY / (double)(maxY + offsetY);
	    }
	}

	System.out.println("offsetX: "+offsetX+" conversionX: "+conversionX);
	System.out.println("offsetY: "+offsetY+" conversionY: "+conversionY);

	nodeEnum = nodes.elements();
	labelEnum = labels.elements();

	// apply conversion factor to all coordinates
	while(nodeEnum.hasMoreElements()) {
	    tmpNode = (Node)nodeEnum.nextElement();
	    tmpNode.x += offsetX;
	    tmpNode.x = (int)((double)tmpNode.x * conversionX);
	    tmpNode.y += offsetY;
	    tmpNode.y = (int)((double)tmpNode.y * conversionY);
	}
	while(labelEnum.hasMoreElements()) {
	    tmpLabel = (Label)labelEnum.nextElement();
	    tmpLabel.x += offsetX;
	    tmpLabel.x = (int)((double)tmpLabel.x * conversionX);
	    tmpLabel.y += offsetY;
	    tmpLabel.y = (int)((double)tmpLabel.y * conversionY);
	}
    }

    /**
     * Assumes all the labels are already at the same X val; 
     *   stagger labels into a specified number of tiers over
     *   the given total space.
     *
     * @param tierCount The number of tiers to divide the labels into.
     * @param spacing The spacing in pixels between the tiers.
     */
    public void staggerLabels(int tierCount, int spacing) {

	Label[] labelList = new Label[labels.size()];

	Enumeration labelEnum = labels.elements();

	for(int i=0; labelEnum.hasMoreElements(); ++i) {
	    labelList[i] = (Label)labelEnum.nextElement();
	}

	Comparator compare = new CompareCoordsByX();
	PlotPathsUtil.quicksort(labelList, compare);

	int newYVal;
	Label tmpLabel;
	for(int i=0; i < labelList.length; i++) {

	    tmpLabel = labelList[i];

	    newYVal = tmpLabel.y - tierCount * spacing;

	    newYVal += (i % tierCount) * spacing;
	    tmpLabel.y = newYVal;
	}
    }


    /** 
     * Sort labels by X coord
    private static void quickSortLabelsByXCoord(Vector labelVec, 
				int[] sortArray) {
	quickSortLabelsByXCoord(labelVec, sortArray, 0, sortArray.length-1);
    }

     * Sort labels by X coord
    private static void quickSortLabelsByXCoord(Vector labelVec, 
				int[] sortArray, int leftEnd, int rightEnd) {

	if(rightEnd - leftEnd <= 0) {
	    return;
	} else if((rightEnd - leftEnd) == 1) {
	    // short circuit for 2 element case
	    if(((Coordinate)labelVec.elementAt(sortArray[leftEnd])).x > 
		    ((Coordinate)labelVec.elementAt(sortArray[rightEnd])).x) {

		// exchange
		int tmp = sortArray[leftEnd];
		sortArray[leftEnd] = sortArray[rightEnd];
		sortArray[rightEnd] = tmp;
	    }
	    return;
	}
	
	int pivot = leftEnd;
	int newLeftEnd = pivot;			// pick first element as pivot 
	int tmp;
	
	for(int i = leftEnd+1; i <= rightEnd; i++) {
	    if(((Coordinate)labelVec.elementAt(sortArray[i])).x  < 
		    ((Coordinate)labelVec.elementAt(sortArray[pivot])).x) {


		newLeftEnd++;

		if(i != newLeftEnd) {

		    // exchange
		    tmp = sortArray[newLeftEnd];
		    sortArray[newLeftEnd] = sortArray[i];
		    sortArray[i] = tmp;
		}
	    }
	}

	tmp = sortArray[newLeftEnd];
	sortArray[newLeftEnd] = sortArray[pivot];
	sortArray[pivot] = tmp;

	quickSortLabelsByXCoord(labelVec, sortArray, leftEnd, newLeftEnd-1);
	quickSortLabelsByXCoord(labelVec, sortArray, newLeftEnd+1, rightEnd);
    }
     */

    /**
     * Add a category to the set.
     */
    public Object addCategory(Object category) {
	return masterCategorySet.addCategory(category);
    }

    /**
     * Add an attribute to inside a category
     */
    public Object addAttribute(Object category, Object attribute, 
			    Object value) {
	return masterCategorySet.addAttribute(category, attribute, value);
    }

    /**
     * Get an attribute from inside a category
     */
    public Object getAttribute(Object category, Object attribute) {
	return masterCategorySet.getAttribute(category, attribute);
    }

    /**
     * Remove an attribute from the set of categories.
     */
    public Object deleteCategory(Object category) {
	return masterCategorySet.deleteCategory(category);
    }

    /**
     * Remove an attribute from inside a category
     */
    public Object deleteAttribute(Object category, Object attribute) {
	return masterCategorySet.deleteAttribute(category, attribute);
    }

    /**
     * Replace an attribute inside a category
     */
    public Object changeAttribute(Object category, Object attribute, 
			    Object value) {

	return masterCategorySet.changeAttribute(category, attribute, value); 
    }
    
    /**
     * Get a list of objects that represent all of the categories
     *    contained in this node's category set.
     */
    public Object[] enumerateCategories() {
	return masterCategorySet.enumerateCategories();
    }

    /** 
     * Get a list of attributes in a given category
     */
    public Object[] enumerateAttributes(Object category) {
	return masterCategorySet.enumerateAttributes(category);
    }

    private Hashtable lines;		// contains Line
    private Hashtable nodes;		// contains Node
    private Hashtable labels;		// contains Label 
    private AttributeSet masterCategorySet = new AttributeSet();
}
