//==========================================================================
//
// Copyright 2002 The Regents of the University of California
// All Rights Reserved
//
// Permission to use, copy, modify and distribute any part of this Makefile
// for educational, research and non-profit purposes, without fee, and
// without a written agreement is hereby granted, provided that the above 
// copyright notice, this paragraph and the following paragraphs appear 
// in all copies.
//
// Those desiring to incorporate this into commercial products or use for
// commercial purposes should contact the
//
//      Technology Transfer Office, University of California, San Diego,
//      9500 Gilman Drive, La Jolla, CA 92093-0910
//      Ph: (619) 534-5815, FAX: (619) 534-7345.
//
// IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS PLOTPATH, EVEN 
// IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF 
// SUCH DAMAGE.
//
// THE PLOTPATH, PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
// UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
// SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY OF 
// CALIFORNIA PLOTPATH NO REPRESENTATIONS AND EXTENDS NO WARRANTIES OF 
// ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED TO, 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
// PURPOSE, OR THAT THE USE OF THE SKITTERTRACES WILL NOT INFRINGE ANY 
// PATENT, TRADEMARK OR OTHER RIGHTS.
//
// PlotPaths is developed by John Gallagher Bradley L. Huffaker at the
// University of California, San Diego under the Cooperative Association
// for Internet Data Analysis (CAIDA) Program. Support for this effort is
// provided by NSF "Internet Atlas" grant ANI-I-9996248, NSF "CAIDA Core"
// NCR-9711092, and by CAIDA memebers.
//==========================================================================
package org.caida.otter;

import java.io.*;
import java.util.Vector;


/**
 *  GetOpts class parses the command line arguments to PlotPaths.
 *  It will set the proper command line options which can
 *  then be accesed by its' proper accessor method.
 *  GetOpts also contains a private status varialbe which will
 *  be set to true(default) if the parsing of the command line
 *  was successful and will be set to false if the command line
 *  parsing failed in some way.
 *  
 *  A description of the command line options can be found in README.
 */
public class GetOpts 
{
    //////////////////////////////////////////////////////
    //    Command Line Options  (and default values)    //
    //////////////////////////////////////////////////////

    private static boolean USE_ALL_FORWARD_PATHS    = false;
    private static boolean DISPLAY_LABELS	    = true;
    //private static String OTTER_OUTFILE		    = "otter.out.odf";
    private static String LIBSEA_OUTFILE            = "graph.libsea.out";
    private static String INPUT_FILE		    = "graph.libsea.in";

    //Not used anymore but keeping it just in case someone
    //tells me otherwise
    private static String NODE_DATA_FILE	    = "nodes.dat";

    private static int SCREEN_X			    = -1;
    private static int SCREEN_Y			    = -1;
    private static int COLUMN_WIDTH		    = 50;
    private static int ROW_SPACING		    = 100;
    private static int NODE_SPACING		    = 25;
    private static int MAX_CHILD_SEARCH_DEPTH	    = -1;
    private static int LABEL_TIERS		    = 1;
    private static int LABEL_SPACING		    = 80;
    private static float MIN_PERCENT_USAGE	    = (float)0.1;
    private static boolean IGNORE_INCOMPLETE_PATHS  = false;
    private static Vector REMOVE_COL_NAMES	    = new Vector();
    private static boolean REMOVE_COLS_SPECIFIED    = false;
    private static int PATH_WEIGHT_BIN_COUNT	    = 5;
    private static String UNKNOWN_NODE_NAME	    = "*";

    // This is the attribute that will be used for column sorting.
    private static String COLUMN_ATTRIBUTE          = "as";
    
    // Flag set to true if user wishs to print Graph Attribute
    // information out to the screen.
    private static boolean LIST_ATTRIBUTES          = false;

    // Flag set to true if user wishs to print available Source
    // information out to the screen.
    private static boolean LIST_SOURCES          = false;

    // Flag set to true if any pre-processing info is needed.
    private static boolean INFO_FLAG                = false;

    // Vector containing the Source Nodes that will lie on the bottom 
    // of the screen
    private static Vector SOURCES                   = new Vector();

    // Gives the status of command line parsing.
    // true = parsing passed, false = parsing failed
    private static boolean Status                   = true;
   
    /**
     *  Constructor for GetOpts.
     *  Sets the Status variable for parsing success or failure
     *  after calling checkArgs with args.
     *  @param args The array of command line arguments.
     */
    public GetOpts(String[] args)
    {
	Status = checkArgs(args);
    }
    
    /**
     *  Accessor function for the private data member USE_ALL_FORWARD_PATHS.
     */
    public boolean get_USE_ALL_FORWARD_PATHS()
    {
	return USE_ALL_FORWARD_PATHS;
    }

    /**
     *  Accessor function for the private data member DISPLAY_LABELS.
     */
    public boolean get_DISPLAY_LABELS()
    {
	return DISPLAY_LABELS;
    }

    /**
     *  Accessor function for the private data member LIBSEA_OUTFILE.
     */
    public String get_LIBSEA_OUTFILE()
    {
	return LIBSEA_OUTFILE;
    }

    /**
     *  Accessor function for the private data member INPUT_FILE.
     */
    public String get_INPUT_FILE()
    {
	return INPUT_FILE;
    }

    /**
     *  Accessor function for the private data member NODE_DATA_FILE.
     */
    public String get_NODE_DATA_FILE()
    {
	return NODE_DATA_FILE;
    }

    /**
     *  Accessor function for the private data member SCREEN_X.
     */
    public int get_SCREEN_X()
    {
	return SCREEN_X;
    }

    /**
     *  Accessor function for the private data member SCREEN_Y.
     */
    public int get_SCREEN_Y()
    {
	return SCREEN_Y;
    }

    /**
     *  Accessor function for the private data member COLUMN_WIDTH.
     */
    public int get_COLUMN_WIDTH()
    {
	return COLUMN_WIDTH;
    }

    /**
     *  Accessor function for the private data member ROW_SPACING.
     */
    public int get_ROW_SPACING()
    {
	return ROW_SPACING;
    }

    /**
     *  Accessor function for the private data member NODE_SPACING.
     */
    public int get_NODE_SPACING()
    {
	return NODE_SPACING;
    }

    /**
     *  Accessor function for the private data member MAX_CHILD_SEARCH_DEPTH.
     */
    public int get_MAX_CHILD_SEARCH_DEPTH()
    {
	return MAX_CHILD_SEARCH_DEPTH;
    }

    /**
     *  Accessor function for the private data member LABEL_TIERS.
     */
    public int get_LABEL_TIERS()
    {
	return LABEL_TIERS;
    }

    /**
     *  Accessor function for the private data member LABEL_SPACING.
     */
    public int get_LABEL_SPACING()
    {
	return LABEL_SPACING;
    }

    /**
     *  Accessor function for the private data member MIN_PERCENT_USAGE.
     */
    public float get_MIN_PERCENT_USAGE()
    {
	return MIN_PERCENT_USAGE;
    }

    /**
     *  Accessor function for the private data member IGNORE_INCOMPLETE_PATHS.
     */
    public boolean get_IGNORE_INCOMPLETE_PATHS()
    {
	return IGNORE_INCOMPLETE_PATHS;
    }

    /**
     *  Accessor function for the private data member REMOVE_COL_NAMES.
     */
    public Vector get_REMOVE_COL_NAMES()
    {
	return REMOVE_COL_NAMES;
    }

    /**
     *  Accessor function for the private data member REMOVE_COLS_SPECIFIED.
     */
    public boolean get_REMOVE_COLS_SPECIFIED()
    {
	return REMOVE_COLS_SPECIFIED;
    }

    /**
     *  Accessor function for the private data member PATH_WEIGHT_BIN_COUNT.
     */
    public int get_PATH_WEIGHT_BIN_COUNT()
    {
	return PATH_WEIGHT_BIN_COUNT;
    }

    /**
     *  Accessor function for the private data member UNKNOWN_NODE_NAME.
     */
    public String get_UNKNOWN_NODE_NAME()
    {
	return UNKNOWN_NODE_NAME;
    }

    /**
     *  Accessor function for the private data member COLUMN_ATTRIBUTE.
     */
    public String get_COLUMN_ATTRIBUTE()
    {
	return COLUMN_ATTRIBUTE;
    }

    /**
     *  Accessor function for the private data member Status. 
     */
    public boolean get_Status()
    {
	return Status;
    }

     /**
     *  Accessor function for the private data member INFO_FLAG. 
     */   
    public boolean get_INFO_FLAG()
    {
	return INFO_FLAG;
    }

     /**
     *  Accessor function for the private data member LIST_ATTRIBUTES. 
     */   
    public boolean get_LIST_ATTRIBUTES()
    {
	return LIST_ATTRIBUTES;
    }

     /**
     *  Accessor function for the private data member LIST_SOURCES. 
     */   
    public boolean get_LIST_SOURCES()
    {
	return LIST_SOURCES;
    }

     /**
     *  Accessor function for the private data member SOURCE. 
     */   
    public Vector get_SOURCES()
    {
	return SOURCES;
    }


    /**
     *  Performs the parsing of the command line arguments.
     *  @param args The array of command line arguments.
     *  @return Boolean true or false. true = no error, false = error.
     */
    private static boolean checkArgs(String[] args)
    {
	
	if(args.length == 0)
	    {
		System.out.println("Use plotpath -h for a list of " + 
				   "command line options.");
		System.out.println("Attempting to load " + INPUT_FILE + 
				   " with default settings.");
		return true;
	    }

	//Check for Graph information qualifiers
	for(int i = 0; i < args.length; i++)
	    {
		if(args[i].equals("--listAttributes"))
		    {
			LIST_ATTRIBUTES = true;
			INFO_FLAG = true;
		    }
		else if(args[i].equals("--listSources"))
		    {
			LIST_SOURCES = true;
			INFO_FLAG = true;
		    }
		else if( args[i].equals("-i") ) 
		    {
			if(i+1 < args.length) 
			    {
				INPUT_FILE = args[i+1];
				i++;
			    } 
			else 
			    {
				printArgsError(args[i], "requires a value");
	
			    }
		    }
	    }

	//If Graph information qualifiers were listed then
	//break out of function so info can be printed.
	if(INFO_FLAG)
	    {
		return true;
	    }

	for(int i = 0; i < args.length; i++)
	    {
		if( args[i].equals("-o") ) {
		    if(i+1 < args.length) {
			LIBSEA_OUTFILE = args[i+1];
			i++;
		    } 
		    else 
			{
			    printArgsError(args[i], "requires a value");
			    return false;
			}
		} 
		else if( args[i].equals("-i") ) 
		    {
			if(i+1 < args.length) 
		    {
			//	INPUT_FILE = args[i+1];
			i++;
		    } 
			//			else 
			//  {
			//printArgsError(args[i], "requires a value");
			//return false;
			//  }
		    } 

		/*  No longer uses the -d option but kept around 
		    just in case it is decided that we want to 
		    use it again.
		
		else if( args[i].equals("-d") ) 
		    {
			if(i+1 < args.length) 
			    {
				NODE_DATA_FILE = args[i+1];
				i++;
			    } 
			else 
			    {
				printArgsError(args[i], "requires a value");
				return false;
			    }
		    } 
		*/

		else if( args[i].equals("--unknownNodeName") ) 
		    {
			if(i+1 < args.length) 
			    {
				UNKNOWN_NODE_NAME = args[i+1];
				i++;
			    } 
			else 
			    {
				printArgsError(args[i], "requires a value");
				return false;
			    }
		    } 
		else if( args[i].equals("--removeColumn") ) 
		    {
			if(i+1 < args.length) 
			    {
				if(!REMOVE_COLS_SPECIFIED) 
				    {
					REMOVE_COL_NAMES.clear();
					REMOVE_COLS_SPECIFIED = true;
				    }
				REMOVE_COL_NAMES.addElement(args[i+1]);
				i++;
			    } else {
				printArgsError(args[i], "requires a value");
				return false;
			    }
		    } 
		else if(args[i].equals("--graphHeight")) 
		    {
			if(i+1 < args.length) 
			    {
				try 
				    {
					SCREEN_Y = Integer.parseInt(args[i+1]);
					i++;
				    } 
				catch (NumberFormatException e) 
				    {
					printArgsError(args[i]+" "+args[i+1], 
						       args[i+1]+" is not an integer");
					return false;
				    }
			    } else 
				{
				    printArgsError(args[i], "requires a value");
				    return false;
				}
		    } 
		else if(args[i].equals("--graphWidth")) 
		    {
			if(i+1 < args.length) 
			    {
				try 
				    {
					SCREEN_X = Integer.parseInt(args[i+1]);
					i++;
				    } 
				catch (NumberFormatException e) 
				    {
					printArgsError(args[i]+" "+args[i+1], 
						       args[i+1]+" is not an integer");
					return false;
				    }
			    } else 
				{
				    printArgsError(args[i], "requires a value");
				    return false;
				}
		    } 
		else if(args[i].equals("--columnWidth")) 
		    {
			if(i+1 < args.length) 
			    {
				try 
				    {
					COLUMN_WIDTH = Integer.parseInt(args[i+1]);
					i++;
				    } 
				catch (NumberFormatException e) 
				    {
					printArgsError(args[i]+" "+args[i+1], 
						       args[i+1]+" is not an integer");
					return false;
				    }
			    } else 
				{
				    printArgsError(args[i], "requires a value");
				    return false;
				}
		    } 
		else if(args[i].equals("--rowHeight")) 
		    {
			if(i+1 < args.length) 
			    {
				try 
				    {
					ROW_SPACING = Integer.parseInt(args[i+1]);
					i++;
				    } 
				catch (NumberFormatException e) 
				    {
					printArgsError(args[i]+" "+args[i+1], 
						       args[i+1]+" is not an integer");
					return false;
				    }
			    } else 
				{
				    printArgsError(args[i], "requires a value");
				    return false;
				}
		    } 
		else if(args[i].equals("--nodeSpacing")) 
		    {
			if(i+1 < args.length) 
			    {
				try 
				    {
					NODE_SPACING = Integer.parseInt(args[i+1]);
					i++;
				    } 
				catch (NumberFormatException e) 
				    {
					printArgsError(args[i]+" "+args[i+1], 
						       args[i+1]+" is not an integer");
					return false;
				    }
			    } 
			else 
			    {
				printArgsError(args[i], "requires a value");
				return false;
			    }
		    } 
		else if(args[i].equals("--noLabels")) 
		    {
			DISPLAY_LABELS = false;
		    } 
		else if(args[i].equals("--labelTiers")) 
		    {
			if(i+1 < args.length) {
			    try 
				{
				    LABEL_TIERS = Integer.parseInt(args[i+1]);
				    i++;
				} 
			    catch (NumberFormatException e) 
				{
				    printArgsError(args[i]+" "+args[i+1], 
						   args[i+1]+ " is not an integer");
				    return false;
				}
			} 
			else 
			    {
				printArgsError(args[i], "requires a value");
				return false;
			    }
		    } 
		else if(args[i].equals("--labelSpacing")) 
		    {
			if(i+1 < args.length) 
			    {
				try 
				    {
					LABEL_SPACING = Integer.parseInt(args[i+1]);
					i++;
				    } 
				catch (NumberFormatException e) 
				    {
					printArgsError(	args[i]+" "+args[i+1], 
							args[i+1]+" is not an integer");
					return false;
				    }
			    }
			else 
			    {
				printArgsError(args[i], "requires a value");
				return false;
			    }
		    } 
		else if(args[i].equals("--maxSpacingSearchDepth")) 
		    {
			if(i+1 < args.length) 
			    {
				try 
				    {
					MAX_CHILD_SEARCH_DEPTH = Integer.parseInt(args[i+1]);
					i++;
				    } 
				catch (NumberFormatException e) 
				    {
					printArgsError(	args[i]+" "+args[i+1], 
							args[i+1]+" is not an integer");
					return false;
				    }
			    } 
			else 
			    {
				printArgsError(args[i], "requires a value");
				return false;
			    }
		    } 
		else if(args[i].equals("--pathWeightBinCount")) 
		    {
			if(i+1 < args.length) 
			    {
				try 
				    {
					PATH_WEIGHT_BIN_COUNT = Integer.parseInt(args[i+1]);
					if(PATH_WEIGHT_BIN_COUNT < 1) 
					    {
						printArgsError( args[i+1], " is not >= 1");
						PATH_WEIGHT_BIN_COUNT = 1;
					    }
					i++;
				    } 
				catch (NumberFormatException e) 
				    {
					printArgsError(	args[i]+" "+args[i+1], 
							args[i+1]+" is not an integer");
					return false;
				    }
			    } 
			else 
			    {
				printArgsError(args[i], "requires a value");
				return false;
			    }
		    } 
		else if(args[i].equals("--minPercentUsage")) 
		    {
			if(i+1 < args.length) 
			    {
				try 
				    {
					MIN_PERCENT_USAGE = Float.parseFloat(args[i+1]);
					i++;
				    } 
				catch (NumberFormatException e) 
				    {
					printArgsError(	args[i]+" "+args[i+1], 
							args[i+1]+"is not a floating "+
							"point value");
					return false;
				    }
			    } 
			else 
			    {
				printArgsError(args[i], "requires a value");
				return false;
			    }
		    } 
		else if(args[i].equals("--useAllForwardPaths")) 
		    {
			USE_ALL_FORWARD_PATHS = true;
		    } 
		else if(args[i].equals("--ignoreIncompletePaths")) 
		    {
			IGNORE_INCOMPLETE_PATHS = true;
		    } 
		else if(args[i].equals("--setAttribute"))
		    {
			COLUMN_ATTRIBUTE = args[i + 1];
			i++;
		    }
		//adding command line selection of sources
		else if(args[i].equals("--setSources"))
		    {
			if(i + 1 < args.length)
			    {
				String tempString = "initialize";
				while(!tempString.equals("-"))
				    {
					try
					    {
						Integer tempInt = new Integer(args[i + 1]);
						if(SOURCES.contains(tempInt))
						    {
							System.out.println("Encountered duplicate source: skipping");
						    }
						else
						    {
							SOURCES.addElement(tempInt);
						    }
						i++;
					    }
					catch(NumberFormatException e)
					    {
						printArgsError(args[i + 1], "needs to be an Integer"); 
						return false;
					    }
					if(i + 1 < args.length)
					    {
						tempString = args[i + 1].substring(0, 1);
					    }
					else
					    {
						//no more tokens must be done
						return true;
					    }
				    }
			    }
			else 
			    {
				printArgsError(args[i], "requires a value");
				return false;
			    }
		    }
		else if(args[i].equals("-h")) 
		    {
			printHelpMessage();
			return false;
		    } 
		else 
		    {
			printArgsError(args[i], "is not a valid argument");
			return false;
		    }
		
	    }
	return true;
    }
    

    /**
     *  Prints the help contents for proper command line usage
     *  with a short description of the command line options.
     */
    private static void printHelpMessage() 
    {
	
	System.out.println("PlotPaths Command line argument summary:\n" +
			   "-h \t\t\tPrint this help message.\n"+
			   "-i \t\t\tSpecify the input file.\n"+
			   "\t\t\t  [default]: graph.libsea.in\n"+
			   "-o \t\t\tSpecify the output file.\n"+
			   "\t\t\t  [default]: graph.libsea\n"+
			   //			   "-d \t\t\tSpecify node data file.\n" +
			   //			   "\t\t\t  [default]: nodes.dat\n"+
			   "--graphHeight \t\tThe output graph height in pixels.\n"+
			   "\t\t\t  [default]: (don't scale graph)\n"+
			   "--graphWidth \t\tThe output graph width in pixels.\n"+
			   "\t\t\t  [default]: (don't scale graph)\n"+
			   "--columnWidth \t\tThe width of a single column in pixels.\n"+
			   "\t\t\t  [default]: 50\n"+
			   "--rowHeight \t\tThe height of a single row in pixels.\n"+
			   "\t\t\t  [default]: 100\n"+
			   "--nodeSpacing \t\tThe spacing between nodes in a single row.\n"+
			   "\t\t\t  [default]: 25\n"+
			   "--labelTiers \t\tThe number of tiers the labels are divided "+
			   "into.\n"+
			   "\t\t\t  [default]: 1\n"+
			   "--labelSpacing \t\tThe vertical space between label tiers.\n"+
			   "\t\t\t  [default]: 80\n"+
			   "--pathWeightBinCount \tThe number of bins to subdivide path "+
			   "weight\n"+
			   "\t\t\t  [default]: 5\n"+
			   "--maxSpacingSearchDepth Maximum depth scanned when spacing "+
			   "nodes.\n"+
			   "\t\t\t  [default]: no limit\n"+
			   "--unknownNodeName \tName of nodes that wern't determined in "+
			   "path\n"+
			   "\t\t\t  [default]: \"*\"\n"+
			   "--removeColumn \t\tSpecify a column to be disolved into its "+
			   "closest \n\t\t\tneighbors. (can be used multiple times)\n"+
			   "--minPercentUsage \tSpecify minimum usage of paths to be "+
			   "considered for \n\t\t\tgraph.\n"+
			   "\t\t\t  [default]: 0.1\n"+
			   "--noLabels \t\tTurn off labels in the output graph.\n"+
			   "--useAllForwardPaths \tSpecify input file as containing all "+
			   "paths as bottom \n\t\t\tto top.\n" +
			   "--setAttribute \tSpecify the Graph File Attribute that you " +
			   "\n\t\t\twant the columns to be sorted on. \n" +
			   "\t\t\t  [default]: \"as\"\n" +
			   "--listAttributes \tPrint information about attributes contained " +
			   "in the file.\n" +
			   "--listSources \tPrint information about availabe sources contained " +
			   "in the file.\n" +
			   "--setSources \tSpecify the Sources that forward Paths will be derived.\n" +
			   "\t\t\t [default]: Will choose first Source in graph file.\n" + 
			   "\t\t\t [Usage]: --setSources <int> <int> ...\n"
			   );
    }


    /**
     *  Prints the command line usage info.  Used if an error
     *  has occured during the parsing of the command line.
     */
    private static void printArgsError() 
    {
	System.out.println("Use PlotPaths -h for a list of command "+
			   "line options");
	System.out.println("Usage : java PlotPaths " + 
			   "[-h] " +
			   "[-i InputFile] " +
			   "[-o OutputFile]\n\t\t" + 
			   //			   "[-d NodeDataFile]\n\t\t" +
			   "[--graphHeight PictureHeight]\n\t\t" + 
			   "[--graphWidth PictureWidth]\n\t\t" +
			   "[--columnWidth ColumnWidth]\n\t\t" +
			   "[--rowHeight RowHeight]\n\t\t" +
			   "[--nodeSpacing NodeSpacing]\n\t\t" +
			   "[--maxSpacingSearchDepth MaxDepth]\n\t\t" +
			   "[--unknownNodeName UnknownNodeName]\n\t\t" +
			   "[--pathWeightBinCount BinCount]\n\t\t" +
			   "[--noLabels]\n\t\t" +
			   "[--labelTiers NumOfTiers]\n\t\t" +
			   "[--labelSpacing labelSpacing]\n\t\t" +
			   "[--removeColumn ColumnToRemove]\n\t\t" +
			   "[--minPercentUsage PercentUsage]\n\t\t" +
			   "[--ignoreIncompletePaths]\n\t\t" +
			   "[--useAllForwardPaths]\n\t\t" +
			   "[--setAttribute]\n\t\t" +
			   "[--listAttributes]\n\t\t" + 
			   "[--listSources]\n\t\t" + 
			   "[--setSources]\n");
    }

    /**
     *  Allows the programmer to customize an error message.
     *  Used if there was an error during the parsing of the
     *  command line arguments.
     *  @param errorString The actual error that has occured.
     *  @param description The description of the error that has occured.
     */
    private static void printArgsError(String errorString, 
				String description) 
    {

	System.out.println("[Invalid Argument]: \"" +
			   errorString+"\" "+description);
	printArgsError();
    }
}		
