//==========================================================================
//
// Copyright 2002 The Regents of the University of California
// All Rights Reserved
//
// Permission to use, copy, modify and distribute any part of this Makefile
// for educational, research and non-profit purposes, without fee, and
// without a written agreement is hereby granted, provided that the above 
// copyright notice, this paragraph and the following paragraphs appear 
// in all copies.
//
// Those desiring to incorporate this into commercial products or use for
// commercial purposes should contact the
//
//      Technology Transfer Office, University of California, San Diego,
//      9500 Gilman Drive, La Jolla, CA 92093-0910
//      Ph: (619) 534-5815, FAX: (619) 534-7345.
//
// IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS PLOTPATH, EVEN 
// IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF 
// SUCH DAMAGE.
//
// THE PLOTPATH, PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
// UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
// SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY OF 
// CALIFORNIA PLOTPATH NO REPRESENTATIONS AND EXTENDS NO WARRANTIES OF 
// ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED TO, 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
// PURPOSE, OR THAT THE USE OF THE SKITTERTRACES WILL NOT INFRINGE ANY 
// PATENT, TRADEMARK OR OTHER RIGHTS.
//
// PlotPaths is developed by John Gallagher Bradley L. Huffaker at the
// University of California, San Diego under the Cooperative Association
// for Internet Data Analysis (CAIDA) Program. Support for this effort is
// provided by NSF "Internet Atlas" grant ANI-I-9996248, NSF "CAIDA Core"
// NCR-9711092, and by CAIDA memebers.
//==========================================================================
package org.caida.otter;

import java.io.*;
import org.caida.libsea.*;

/**
 *  This class will read in a graph file as defined in the LibSea
 *  graph library.  It will also write out the file in the graph
 *  file format.
 */
public class GraphIO
{
    /**
     *  @param filename The file that contains the graph data
     *  @returns A graph object
     */
    public Graph loadGraph(String filename)
    {
	Graph retval = null;
	try
	{
	    ASCIIInputStreamReader reader =
		new ASCIIInputStreamReader(new FileInputStream(filename));

	    GraphBuilder builder = GraphFactory.makeImmutableGraph();
	    GraphFileLexer lexer = new GraphFileLexer(reader);
	    GraphFileParser parser = new GraphFileParser(lexer);
	    parser.file(builder);

	    retval = builder.endConstruction();
	}
	catch (FileNotFoundException e)
	{
	    String msg = "Error opening file `" + filename + "': "
		+ e.toString();
	    System.err.println(msg);
	}
	catch (antlr.ANTLRException e) // thrown by lexer or parser
	{
	    // NOTE: ANTLRException.toMessage() doesn't include position.
	    String msg = "Error parsing file `" + filename + "': "
		+ e.toString();
	    System.err.println(msg);
	}

	return retval;
    }



    /**
     *  @param filename The filename that will be written.
     *  @param graph The graph that you want written to the file.       
     *  @param isVerbose Whether you want identifiers placed in graph file.
     */
    public void writeGraph(String filename, Graph graph, boolean isVerbose)
    {
	try
	    {
		Writer m_writer = new BufferedWriter(new OutputStreamWriter
						 (new FileOutputStream(filename)));
		new GraphFileWriter(m_writer, graph, isVerbose, false).write();
	    }
	catch (IOException e)
	    {
		String msg = "IOException encountered: " + e.toString();
		System.err.println(msg);
	    }
    }

}
