//==========================================================================
//
// Copyright 2002 The Regents of the University of California
// All Rights Reserved
//
// Permission to use, copy, modify and distribute any part of this Makefile
// for educational, research and non-profit purposes, without fee, and
// without a written agreement is hereby granted, provided that the above 
// copyright notice, this paragraph and the following paragraphs appear 
// in all copies.
//
// Those desiring to incorporate this into commercial products or use for
// commercial purposes should contact the
//
//      Technology Transfer Office, University of California, San Diego,
//      9500 Gilman Drive, La Jolla, CA 92093-0910
//      Ph: (619) 534-5815, FAX: (619) 534-7345.
//
// IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS PLOTPATH, EVEN 
// IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF 
// SUCH DAMAGE.
//
// THE PLOTPATH, PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
// UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
// SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY OF 
// CALIFORNIA PLOTPATH NO REPRESENTATIONS AND EXTENDS NO WARRANTIES OF 
// ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED TO, 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
// PURPOSE, OR THAT THE USE OF THE SKITTERTRACES WILL NOT INFRINGE ANY 
// PATENT, TRADEMARK OR OTHER RIGHTS.
//
// PlotPaths is developed by John Gallagher Bradley L. Huffaker at the
// University of California, San Diego under the Cooperative Association
// for Internet Data Analysis (CAIDA) Program. Support for this effort is
// provided by NSF "Internet Atlas" grant ANI-I-9996248, NSF "CAIDA Core"
// NCR-9711092, and by CAIDA memebers.
//==========================================================================
package org.caida.otter;

import java.text.*;
import org.caida.libsea.*;

/**
 *  Class to gain information from LibSea graph file.  Useful for
 *  pre-proccessing activites where user may need to know something
 *  about the graph file so they can accurately define parameters
 *  for proccessing.
 */
public class GraphInfo
{
    private TextTools t                           = new TextTools();



    /**
     *  Retrieves Attribute Information about the Graph file and 
     *  prints it out to the screen in a columated format.
     *  @param LibSea_Graph The graph file that the info will be retreived from.
     */
    public void printAttributeInfo(Graph LibSea_Graph)
    {
	AttributeDefinitionIterator AttributeMainIterator = LibSea_Graph.getAttributeDefinitions();
	String Node, Link, Path;
	int numNodes = LibSea_Graph.getNumNodes();
	int numLinks = LibSea_Graph.getNumLinks();
	int numPaths = LibSea_Graph.getNumPaths();
	int tempNode, tempLink, tempPath;

	System.out.println("======================== Graph Attributes =========================");

	while(!AttributeMainIterator.atEnd())
	    {
		ValueType AttributeType = AttributeMainIterator.getType();
		AttributesByAttributeIterator NodeIterator = AttributeMainIterator.getNodeAttributes();
		AttributesByAttributeIterator LinkIterator = AttributeMainIterator.getLinkAttributes();
		AttributesByAttributeIterator PathIterator = AttributeMainIterator.getPathAttributes();

		Node = Link = Path = "True";
		tempNode = tempLink = tempPath = 0;

		//Check to see whether there are any values for Nodes.
		if(NodeIterator.atEnd())
		    {
			Node = "False";
		    }
		//Check to see whether there are any values for Links.		
		if(LinkIterator.atEnd())
		    {
			Link = "False";
		    }
		//Check to see whether there are any values for Paths.	
		if(PathIterator.atEnd())
		    {
			Path = "False";
		    }
		
		if(Node == "True")
		    {
			while(!NodeIterator.atEnd())
			    {
				tempNode++;
				NodeIterator.advance();
			    }
		    }
		
		if(Link == "True")
		    {
			while(!LinkIterator.atEnd())
			    {
				tempLink++;
				LinkIterator.advance();
			    }
		    }

		if(Path == "True")
		    {
			while(!PathIterator.atEnd())
			    {
				tempPath++;
				PathIterator.advance();
			    }
		    }

		// Print out the attribute information
		StringBuffer b = new StringBuffer(0);
		b.append("Name: " +  AttributeMainIterator.getName());
		try
		    {
			b = t.addWhiteSpace(b,40);
		    }
		catch(NeedMoreWhiteSpace e)
		    {
			System.out.println(e);
			System.exit(1);
		    }
		b.append("Type: " + AttributeType.getName());
		try
		    {
			b = t.addWhiteSpace(b,70);
		    }
		catch(NeedMoreWhiteSpace e)
		    {
			System.out.println(e);
			System.exit(1);
		    }
		b.append("Default: " + AttributeMainIterator.getDefault());
		System.out.println(b.toString());
		
		StringBuffer c = new StringBuffer(0);
		if(Node == "True")
		    {
			c.append("Node Values: " + (tempNode == numNodes ? "Complete" : "Incomplete"));
		    }
		else
		    {
			c.append("Node Values: Empty");
		    }
		try
		    {
			c = t.addWhiteSpace(c,40);
		    }
		catch(NeedMoreWhiteSpace e)
		    {
			System.out.println(e);
			System.exit(1);
		    }
		if(Link == "True")
		    {
			c.append("Link Values: " + (tempLink == numLinks ? "Complete" : "Incomplete"));
		    }
		else
		    {
			c.append("Link Values: Empty");
		    }
		try
		    {
			c = t.addWhiteSpace(c,70);
		    }
		catch(NeedMoreWhiteSpace e)
		    {
			System.out.println(e);
			System.exit(1);
		    }
		if(Path == "True")
		    {
			c.append("Path Values: " + (tempPath == numPaths ? "Complete" : "Incomplete"));
		    }
		else
		    {
			c.append("Path Values: Empty");
		    }
		System.out.println(c.toString());		
		System.out.println("----------");

		AttributeMainIterator.advance();
	    }
    }

    public void printAvailableSources(Graph LibSea_Graph)
    {
	AttributeDefinitionIterator AttributeMainIterator = LibSea_Graph.getAttributeDefinition("source");
	AttributesByAttributeIterator NodeIterator = AttributeMainIterator.getNodeAttributes();
	
	System.out.println("======================== Available Sources =========================");
	while(!NodeIterator.atEnd())
	    {
		ValueIterator SourceValues = NodeIterator.getAttributeValues();
		
		if(SourceValues.getBooleanValue())
		    {
			System.out.println("Source: " + NodeIterator.getObjectID());
		    }
		NodeIterator.advance();
	    }
    }
}
