//==========================================================================
//
// Copyright 2002 The Regents of the University of California
// All Rights Reserved
//
// Permission to use, copy, modify and distribute any part of this Makefile
// for educational, research and non-profit purposes, without fee, and
// without a written agreement is hereby granted, provided that the above 
// copyright notice, this paragraph and the following paragraphs appear 
// in all copies.
//
// Those desiring to incorporate this into commercial products or use for
// commercial purposes should contact the
//
//      Technology Transfer Office, University of California, San Diego,
//      9500 Gilman Drive, La Jolla, CA 92093-0910
//      Ph: (619) 534-5815, FAX: (619) 534-7345.
//
// IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS PLOTPATH, EVEN 
// IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF 
// SUCH DAMAGE.
//
// THE PLOTPATH, PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
// UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
// SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY OF 
// CALIFORNIA PLOTPATH NO REPRESENTATIONS AND EXTENDS NO WARRANTIES OF 
// ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED TO, 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
// PURPOSE, OR THAT THE USE OF THE SKITTERTRACES WILL NOT INFRINGE ANY 
// PATENT, TRADEMARK OR OTHER RIGHTS.
//
// PlotPaths is developed by John Gallagher Bradley L. Huffaker at the
// University of California, San Diego under the Cooperative Association
// for Internet Data Analysis (CAIDA) Program. Support for this effort is
// provided by NSF "Internet Atlas" grant ANI-I-9996248, NSF "CAIDA Core"
// NCR-9711092, and by CAIDA memebers.
//==========================================================================
package org.caida.otter;
    
public class Line {
    public String name = null;
    public Coordinate coord1 = null;
    public Coordinate coord2 = null;
    public AttributeSet attribSet = new AttributeSet();

    public Line(Coordinate coord1, Coordinate coord2, String name) {
	this.coord1 = coord1;
	this.coord2 = coord2;
	this.name = name;
    }

    public String getName() {
	return name;
    }

    public Coordinate getCoord1() {
	return coord1;
    }
    
    public Coordinate getCoord2() {
	return coord2;
    }


    
    /**
     * Add a category to the set.
     */
    public Object addCategory(Object category) {
	return attribSet.addCategory(category);
    }

    /**
     * Add an attribute to inside a category
     */
    public Object addAttribute(Object category, Object attribute, 
			    Object value) {
	return attribSet.addAttribute(category, attribute, value);
    }

    /**
     * Get an attribute from inside a category
     */
    public Object getAttribute(Object category, Object attribute) {
	return attribSet.getAttribute(category, attribute);
    }

    /**
     * Remove an attribute from the set of categories.
     */
    public Object deleteCategory(Object category) {
	return attribSet.deleteCategory(category);
    }

    /**
     * Remove an attribute from inside a category
     */
    public Object deleteAttribute(Object category, Object attribute) {
	return attribSet.deleteAttribute(category, attribute);
    }

    /**
     * Replace an attribute inside a category
     */
    public Object changeAttribute(Object category, Object attribute, 
			    Object value) {

	return attribSet.changeAttribute(category, attribute, value); 
    }
    
    /**
     * Get a list of objects that represent all of the categories
     *    contained in this node's category set.
     */
    public Object[] enumerateCategories() {
	return attribSet.enumerateCategories();
    }

    /** 
     * Get a list of attributes in a given category
     */
    public Object[] enumerateAttributes(Object category) {
	return attribSet.enumerateAttributes(category);
    }
}
