//==========================================================================
//
// Copyright 2002 The Regents of the University of California
// All Rights Reserved
//
// Permission to use, copy, modify and distribute any part of this Makefile
// for educational, research and non-profit purposes, without fee, and
// without a written agreement is hereby granted, provided that the above 
// copyright notice, this paragraph and the following paragraphs appear 
// in all copies.
//
// Those desiring to incorporate this into commercial products or use for
// commercial purposes should contact the
//
//      Technology Transfer Office, University of California, San Diego,
//      9500 Gilman Drive, La Jolla, CA 92093-0910
//      Ph: (619) 534-5815, FAX: (619) 534-7345.
//
// IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS PLOTPATH, EVEN 
// IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF 
// SUCH DAMAGE.
//
// THE PLOTPATH, PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
// UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
// SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY OF 
// CALIFORNIA PLOTPATH NO REPRESENTATIONS AND EXTENDS NO WARRANTIES OF 
// ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED TO, 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
// PURPOSE, OR THAT THE USE OF THE SKITTERTRACES WILL NOT INFRINGE ANY 
// PATENT, TRADEMARK OR OTHER RIGHTS.
//
// PlotPaths is developed by John Gallagher Bradley L. Huffaker at the
// University of California, San Diego under the Cooperative Association
// for Internet Data Analysis (CAIDA) Program. Support for this effort is
// provided by NSF "Internet Atlas" grant ANI-I-9996248, NSF "CAIDA Core"
// NCR-9711092, and by CAIDA memebers.
//==========================================================================
package org.caida.otter;

public class LineSegment {

    /**
     * Create a line segment from these coordinates
     */
    public LineSegment(int x1, int y1, int x2, int y2) {
	this.x1 = x1;
	this.x2 = x2;
	this.y1 = y1;
	this.y2 = y2;

	calculateSlopeAndIntercept();
    }

    /**
     * Create a line segment from these coordinates
     */
    public LineSegment(float x1, float y1, float x2, float y2) {
	this.x1 = x1;
	this.x2 = x2;
	this.y1 = y1;
	this.y2 = y2;

	calculateSlopeAndIntercept();
    }


    /**
     * Override default toString to print Coordinates of line
     *   segment to the screen
     */
    public String toString() {
	return("LineSegment("+x1+","+y1+","+x2+","+y2+")");
    }

    /**
     * Test of the current line segment intersects with the argument.
     *    Returns a the boolean result of the intersection test.

     * @return result of intersection test -> true = overlap, 
     *	  false = no overlap.
     */
    public boolean doesLineIntersect(LineSegment otherLine) {

	boolean return_value = false;

	if(this.isVertical && otherLine.isVertical) {
	    if(this.x1 == otherLine.x1) {

		// check for segment overlap
		if(checkFor1DContainment(   this.y1, 
					    this.y2, 
					    otherLine.y1,
					    otherLine.y2)) {

		    return_value = true;
		}
	    }
	} else if(this.isVertical) {

	    // extrapolate a point on vertical line and 
	    // see if it falls between y coords
	    
	    float extrapolate_y;

	    extrapolate_y = (otherLine.slope * this.x1) + otherLine.intercept;

	    if(checkFor1DContainment(extrapolate_y, this.y1, this.y2)) {

		if(checkFor1DContainment(   this.x1, 
					    this.x2, 
					    otherLine.x1, 
					    otherLine.x2)) {
		    return_value = true;
		}
	    }

	} else if(otherLine.isVertical) {

	    // extrapolate a point on vertical line and see 
	    // if it falls between y coords
	    
	    float extrapolate_y;
	    extrapolate_y = (this.slope * otherLine.x1) + this.intercept;

	    if(checkFor1DContainment(	extrapolate_y,
					otherLine.y1,
					otherLine.y2)) {
	    /*
	    if(extrapolate_y <= otherLine.y2 && 
		    extrapolate_y >= otherLine.y1 ||
		    extrapolate_y <= otherLine.y1 && 
		    extrapolate_y >= otherLine.y2) {
	    */

		if(checkFor1DContainment(   this.x1, 
					    this.x2, 
					    otherLine.x1, 
					    otherLine.x2)) {

		    return_value = true;
		}
	    }

	} else {
	    if(this.slope == otherLine.slope) {

		// are they the same line?
		if(this.intercept == otherLine.intercept) { 

		    // check for segment overlap
		    if(checkFor1DContainment(	this.y1, 
						this.y2, 
						otherLine.y1, 
						otherLine.y2)) {

			return_value = true;
		    }
		} else {

		    // are parallel but don't overlap
		
		}
	    } else {

		// find y2-y1 by subtracting the two line equations
		//    ( difference in y is 0 -> they intersect at that point

		float extrapolate_intersect_x = 
			    (this.intercept - otherLine.intercept) / 
			    (otherLine.slope - this.slope);
		float extrapolate_intersect_y = 
			    this.slope * extrapolate_intersect_x + 
			    this.intercept;
				
		if(checkFor1DContainment(	extrapolate_intersect_x, 
						otherLine.x1, 
						otherLine.x2) &&
			checkFor1DContainment(	extrapolate_intersect_y, 
						otherLine.y1, 
						otherLine.y2) &&
			checkFor1DContainment(	extrapolate_intersect_x, 
						this.x1, 
						this.x2) &&
			checkFor1DContainment(	extrapolate_intersect_y, 
						this.y1, 
						this.y2)) {

		    return_value = true;
		}
	    }
	}

	return return_value;
    }

    /**
     * Calculate the slope and intercept, or just set the isVertical 
     * flag when slope can't be calculated.
     */
    private void calculateSlopeAndIntercept() {

	// y = slope * x + intercept

	if(x2-x1 == 0) {
	    isVertical = true;
	    //System.out.println("Line is vertical");

	} else {

	    // m = (y2-y1) / (x2-x1)
	    slope = (float)(y2-y1) / (float)(x2-x1);
	    
	    intercept = y1 - (slope * x1);
	}
    }


    /**
     * When you only want to check a single point against a line segment, this
     *   privides a shorter call to the four argument version of the routine.
     */
    private boolean checkFor1DContainment(float numA1, float numB1, 
					    float numB2) {

	return checkFor1DContainment(numA1, numA1, numB1, numB2);
    }

    /**
     * Take two line segments and return whether or not they overlap.  
     *   The segments are given using one dimension.  numA1-numA2 
     *   define the first segment, numB1-numB2 define the second segment.
     */
    private boolean checkFor1DContainment(float numA1, float numA2, 
					    float numB1, float numB2) {

	boolean return_value = false;

	// check for segment overlap
	if(numB1 < numB2) { 

	    if(!(numA1 < numB1 && numA2 < numB1) && 
			!(numA1 > numB2 && numA2 > numB2)) {

		return_value = true;
	    }

	} else {

	    if(!(numA1 < numB2 && numA2 < numB2) && 
			!(numA1 > numB1 && numA2 > numB1)) {

		return_value = true;
	    }
	}

	return return_value;
    }

    /**
     * return x1
     */
    public float getX1() {
	return x1;
    }

    /**
     * return y1
     */
    public float getY1() {
	return y1;
    }

    /**
     * return x2
     */
    public float getX2() {
	return x2;
    }

    /**
     * return y2
     */
    public float getY2() {
	return y2;
    }


    // x1 <= x2 && y1 <= y2
    private float x1, y1, x2, y2;
    private float slope, intercept;
    private boolean isVertical;
}

