//==========================================================================
//
// Copyright 2002 The Regents of the University of California
// All Rights Reserved
//
// Permission to use, copy, modify and distribute any part of this Makefile
// for educational, research and non-profit purposes, without fee, and
// without a written agreement is hereby granted, provided that the above 
// copyright notice, this paragraph and the following paragraphs appear 
// in all copies.
//
// Those desiring to incorporate this into commercial products or use for
// commercial purposes should contact the
//
//      Technology Transfer Office, University of California, San Diego,
//      9500 Gilman Drive, La Jolla, CA 92093-0910
//      Ph: (619) 534-5815, FAX: (619) 534-7345.
//
// IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS PLOTPATH, EVEN 
// IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF 
// SUCH DAMAGE.
//
// THE PLOTPATH, PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
// UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
// SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY OF 
// CALIFORNIA PLOTPATH NO REPRESENTATIONS AND EXTENDS NO WARRANTIES OF 
// ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED TO, 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
// PURPOSE, OR THAT THE USE OF THE SKITTERTRACES WILL NOT INFRINGE ANY 
// PATENT, TRADEMARK OR OTHER RIGHTS.
//
// PlotPaths is developed by John Gallagher Bradley L. Huffaker at the
// University of California, San Diego under the Cooperative Association
// for Internet Data Analysis (CAIDA) Program. Support for this effort is
// provided by NSF "Internet Atlas" grant ANI-I-9996248, NSF "CAIDA Core"
// NCR-9711092, and by CAIDA memebers.
//==========================================================================
package org.caida.otter;

import java.util.Comparator;
import java.util.Vector;

public class PlotPathsUtil
{
    /**
     * Generic quicksort that sorts a list of things
     *
     * @param objectList list of things to sort
     * @param comparator object that knows how to sort the things that 
     *         are given in the objectList.
     */
    public static void quicksort(   Object[] objectList, 
				    Comparator comparator) {

	quicksort(  objectList, 
		    comparator, 
		    0, 
		    objectList.length-1);

    }
    public static void quicksort(   Object[] objectList, 
				    Comparator comparator,
				    int leftEnd,
				    int rightEnd) {
	Object tmp;

	if(rightEnd - leftEnd <= 0) {
	    return;
	} else if((rightEnd - leftEnd) == 1) {
	    // short circuit for 2 element case
	    if(comparator.compare(  objectList[leftEnd],
				    objectList[rightEnd]) > 0) {

		// exchange
		tmp = objectList[leftEnd];
		objectList[leftEnd] = objectList[rightEnd];
		objectList[rightEnd] = tmp;
	    }
	    return;
	}

	int pivot = leftEnd;            // pick first element as pivot
	int newLeftEnd = pivot;

	for(int i = leftEnd+1; i <= rightEnd; i++) {
	    if(comparator.compare(  objectList[i],
				    objectList[pivot]) < 0) {

		newLeftEnd++;

		if(i != newLeftEnd) {

		    // exchange
		    tmp = objectList[newLeftEnd];
		    objectList[newLeftEnd] = objectList[i];
		    objectList[i] = tmp;

		}
	    }
	}

	tmp = objectList[newLeftEnd];
	objectList[newLeftEnd] = objectList[pivot];
	objectList[pivot] = tmp;

	quicksort(objectList, comparator, leftEnd, newLeftEnd-1);
	quicksort(objectList, comparator, newLeftEnd+1, rightEnd);
    }


    /**
     * Generic quicksort that sorts a list of objects by a foreign key.
     *    The subroutine ends up sorting foreignKeyList, and each
     *    substitution is mirrored in objectList.
     *
     * @param objectList the objects to sort.
     * @param foreignKeyList the values to sort the objects by.  For example
     *         the third index into foreignKeyList corresponds to the value
     *         given to the third object in objectList.  objectList is
     *         sorted such that if it was swappin with its foreign key,
     *         the foreign keys would be in ascending order.
     *
     */
    public static void quicksortByForeignKey(	Object[] objectList,
						int[] foreignKeyList) {

	quicksortByForeignKey(	objectList, 
				foreignKeyList, 
				0, 
				objectList.length-1);
    }
    public static void quicksortByForeignKey(	Object[] objectList,
						int[] foreignKeyList,
						int leftEnd,
						int rightEnd) {

	int tmp;
	Object tmpObj;

	if(rightEnd - leftEnd <= 0) {
	    return;
	} else if((rightEnd - leftEnd) == 1) {
	    // short circuit for 2 element case
	    if(foreignKeyList[leftEnd] > foreignKeyList[rightEnd]) {

		// exchange
		tmp = foreignKeyList[leftEnd];
		foreignKeyList[leftEnd] = foreignKeyList[rightEnd];
		foreignKeyList[rightEnd] = tmp;

		tmpObj = objectList[leftEnd];
		objectList[leftEnd] = objectList[rightEnd];
		objectList[rightEnd] = tmpObj;
	    }
	    return;
	}

	int pivot = leftEnd;            // pick first element as pivot
	int newLeftEnd = pivot;

	for(int i = leftEnd+1; i <= rightEnd; i++) {
	    if(foreignKeyList[i] < foreignKeyList[pivot]) {

		newLeftEnd++;

		if(i != newLeftEnd) {

		    // exchange
		    tmp = foreignKeyList[newLeftEnd];
		    foreignKeyList[newLeftEnd] = foreignKeyList[i];
		    foreignKeyList[i] = tmp;

		    tmpObj = objectList[newLeftEnd];
		    objectList[newLeftEnd] = objectList[i];
		    objectList[i] = tmpObj;

		}
	    }
	}

	tmp = foreignKeyList[newLeftEnd];
	foreignKeyList[newLeftEnd] = foreignKeyList[pivot];
	foreignKeyList[pivot] = tmp;

	tmpObj = objectList[newLeftEnd];
	objectList[newLeftEnd] = objectList[pivot];
	objectList[pivot] = tmpObj;

	quicksortByForeignKey(	objectList, foreignKeyList, 
				leftEnd, newLeftEnd-1);
	quicksortByForeignKey(	objectList, foreignKeyList, 
				newLeftEnd+1, rightEnd);
    }

    /**
     * Generic quicksort that sorts a list of objects by a foreign key. 
     *    The subroutine ends up sorting foreignKeyList, and each
     *    substitution is mirrored in objectList.
     *
     * @param objectList the objects to sort.
     * @param foreignKeyList the values to sort the objects by.  For example
     *         the third index into foreignKeyList corresponds to the value
     *         given to the third object in objectList.  objectList is
     *         sorted such that if it was swappin with its foreign key,
     *         the foreign keys would be in ascending order.
     *
     */
    public static void quicksortByForeignKey(	int[] objectList,
						int[] foreignKeyList) {

	quicksortByForeignKey(	objectList, 
				foreignKeyList, 
				0, 
				objectList.length-1);
    }
    public static void quicksortByForeignKey(	int[] objectList,
						int[] foreignKeyList,
						int leftEnd,
						int rightEnd) {
	int tmp;

	if(rightEnd - leftEnd <= 0) {
	    return;
	} else if((rightEnd - leftEnd) == 1) {
	    // short circuit for 2 element case
	    if(foreignKeyList[leftEnd] > foreignKeyList[rightEnd]) {

		// exchange
		tmp = foreignKeyList[leftEnd];
		foreignKeyList[leftEnd] = foreignKeyList[rightEnd];
		foreignKeyList[rightEnd] = tmp;

		tmp = objectList[leftEnd];
		objectList[leftEnd] = objectList[rightEnd];
		objectList[rightEnd] = tmp;
	    }
	    return;
	}

	int pivot = leftEnd;            // pick first element as pivot
	int newLeftEnd = pivot;

	for(int i = leftEnd+1; i <= rightEnd; i++) {
	    if(foreignKeyList[i] < foreignKeyList[pivot]) {


		newLeftEnd++;

		if(i != newLeftEnd) {

		    // exchange
		    tmp = foreignKeyList[newLeftEnd];
		    foreignKeyList[newLeftEnd] = foreignKeyList[i];
		    foreignKeyList[i] = tmp;

		    tmp = objectList[newLeftEnd];
		    objectList[newLeftEnd] = objectList[i];
		    objectList[i] = tmp;

		}
	    }
	}

	tmp = foreignKeyList[newLeftEnd];
	foreignKeyList[newLeftEnd] = foreignKeyList[pivot];
	foreignKeyList[pivot] = tmp;

	tmp = objectList[newLeftEnd];
	objectList[newLeftEnd] = objectList[pivot];
	objectList[pivot] = tmp;

	quicksortByForeignKey(	objectList, foreignKeyList, 
				leftEnd, newLeftEnd-1);
	quicksortByForeignKey(objectList, foreignKeyList, 
				newLeftEnd+1, rightEnd);
    }

    /**
     * Generic quicksort that sorts a list of objects by a foreign key.
     *    The subroutine ends up sorting foreignKeyList, and each
     *    substitution is mirrored in objectList.
     *
     * @param objectList the objects to sort.
     * @param foreignKeyList the values to sort the objects by.  For example
     *         the third index into foreignKeyList corresponds to the value
     *         given to the third object in objectList.  objectList is
     *         sorted such that if it was swappin with its foreign key,
     *         the foreign keys would be in ascending order.
     *
     */
    public static void quicksortByForeignKey(	Object[] objectList,
						Object[] foreignKeyList,
						Comparator comparator) {

	quicksortByForeignKey(	objectList,
				foreignKeyList,
				comparator,
				0,
				objectList.length-1);
    }
    private static void quicksortByForeignKey(	Object[] objectList,
						Object[] foreignKeyList,
						Comparator comparator,
						int leftEnd,
						int rightEnd) {
	Object tmp;

	if(rightEnd - leftEnd <= 0) {
	    return;
	} else if((rightEnd - leftEnd) == 1) {
	    // short circuit for 2 element case
	    if(comparator.compare(  foreignKeyList[leftEnd], 
				    foreignKeyList[rightEnd]) > 0) {

		// exchange
		tmp = foreignKeyList[leftEnd];
		foreignKeyList[leftEnd] = foreignKeyList[rightEnd];
		foreignKeyList[rightEnd] = tmp;

		tmp = objectList[leftEnd];
		objectList[leftEnd] = objectList[rightEnd];
		objectList[rightEnd] = tmp;
	    }
	    return;
	}

	int pivot = leftEnd;            // pick first element as pivot
	int newLeftEnd = pivot;

	for(int i = leftEnd+1; i <= rightEnd; i++) {
	    if(comparator.compare(  foreignKeyList[i], 
				    foreignKeyList[pivot]) < 0) {

		newLeftEnd++;

		if(i != newLeftEnd) {

		    // exchange
		    tmp = foreignKeyList[newLeftEnd];
		    foreignKeyList[newLeftEnd] = foreignKeyList[i];
		    foreignKeyList[i] = tmp;

		    tmp = objectList[newLeftEnd];
		    objectList[newLeftEnd] = objectList[i];
		    objectList[i] = tmp;

		}
	    }
	}

	tmp = foreignKeyList[newLeftEnd];
	foreignKeyList[newLeftEnd] = foreignKeyList[pivot];
	foreignKeyList[pivot] = tmp;

	tmp = objectList[newLeftEnd];
	objectList[newLeftEnd] = objectList[pivot];
	objectList[pivot] = tmp;

	quicksortByForeignKey(	objectList, foreignKeyList, comparator,
				leftEnd, newLeftEnd-1);
	quicksortByForeignKey(	objectList, foreignKeyList, comparator,
				newLeftEnd+1, rightEnd);
    }

    /**
     * Check a vector to see if it contains duplucates, and remove them if
     * it does.
     *
     * @param checkVector vector to check for duplicates
     */
    public static void removeDuplicatesFromVect(Vector checkVector) {
	for(int i = 0; i < checkVector.size(); i++) {
	    for(int j = i + 1; j < checkVector.size(); j++) {
		if((checkVector.elementAt(i)).equals(
			    checkVector.elementAt(j))) {
		    // pull a duplicate
		    checkVector.remove(j);
		    j--;
		}
	    }
	}
    }
}
