//==========================================================================
//
// Copyright 2002 The Regents of the University of California
// All Rights Reserved
//
// Permission to use, copy, modify and distribute any part of this Makefile
// for educational, research and non-profit purposes, without fee, and
// without a written agreement is hereby granted, provided that the above 
// copyright notice, this paragraph and the following paragraphs appear 
// in all copies.
//
// Those desiring to incorporate this into commercial products or use for
// commercial purposes should contact the
//
//      Technology Transfer Office, University of California, San Diego,
//      9500 Gilman Drive, La Jolla, CA 92093-0910
//      Ph: (619) 534-5815, FAX: (619) 534-7345.
//
// IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
// FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS PLOTPATH, EVEN 
// IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF 
// SUCH DAMAGE.
//
// THE PLOTPATH, PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
// UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
// SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY OF 
// CALIFORNIA PLOTPATH NO REPRESENTATIONS AND EXTENDS NO WARRANTIES OF 
// ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED TO, 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
// PURPOSE, OR THAT THE USE OF THE SKITTERTRACES WILL NOT INFRINGE ANY 
// PATENT, TRADEMARK OR OTHER RIGHTS.
//
// PlotPaths is developed by John Gallagher Bradley L. Huffaker at the
// University of California, San Diego under the Cooperative Association
// for Internet Data Analysis (CAIDA) Program. Support for this effort is
// provided by NSF "Internet Atlas" grant ANI-I-9996248, NSF "CAIDA Core"
// NCR-9711092, and by CAIDA memebers.
//==========================================================================
package org.caida.otter;

import java.text.*;

/**
 *  Added features to manipulate text and strings.
 */
public class TextTools
{
    /**
     *  Allows the ability to add white space to a string buffer so one
     *  can specify the columns and have them be unified accross stdout.
     *  @param a The stringbuffer that will have white space added to it.
     *  @param position The position of the next column.
     *  @return The modified stringbuffer.
     *  @throws NeedMoreWhiteSpace Thrown if the next column position is
     *  less than the current length of the stringbuffer.
     */
    public StringBuffer addWhiteSpace(StringBuffer a, int position) throws NeedMoreWhiteSpace
    {
	int temp = 0;
	temp = position - a.length();
	if( temp < 0 )
	    {
		throw new NeedMoreWhiteSpace("A string token is too long and more " +
					     "whitespace is needed to seperate into columns");
	    }

	for(int i = 0; i < temp - 1; i++)
	    {
		a.append(" ");
	    }
	
	return a;
    }
}
